@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005 Sun Microsystems Inc.
+# Copyright 2005-2006 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -32,11 +32,7 @@ import Accessibility__POA
 
 import debug
 import rolenames
-
-# If True, attempt to cache accessible object information locally.
-# If False, always use a CORBA call to get the object information.
-#
-CACHE_VALUES = True
+import settings
 
 class Event:
     """Converts the source of an event to an Accessible object.  We
@@ -51,7 +47,7 @@ class Event:
         """Returns the any_data field, compensating for any
         differences between pre-1.7.0 and post-1.7.0 implementations
         of the AT-SPI."""
-        
+
         # [[[TODO: WDW - HACK because AT-SPI 1.7.0 has
         # introduced a binary incompatibility where the
         # "any_data" of the event has changed from being a
@@ -69,7 +65,7 @@ class Event:
             return e.any_data.value().any_data
         else:
             return e.any_data
-       
+
     getAnyData = staticmethod(getAnyData)
 
     def __init__(self, e=None):
@@ -78,7 +74,7 @@ class Event:
             self.type     = e.type
             self.detail1  = e.detail1
             self.detail2  = e.detail2
-            
+
             # If were talking to AT-SPI 1.7.0 or greater, we can get the
             # application information right away because it is tucked in
             # the EventDetails data new for 1.7.0.
@@ -114,22 +110,22 @@ class Registry:
     def __init__(self):
 
         # The "Borg" singleton model - ensures we're really
-	# only connecting to the registry once.
-	#
-	self.__dict__ = self.__sharedState
+        # only connecting to the registry once.
+        #
+        self.__dict__ = self.__sharedState
         self.__instanceCount += 1
-	if not self.__dict__.has_key("registry"):
-	    self.registry = bonobo.get_object(
-		"OAFIID:Accessibility_Registry:1.0",
-        	"Accessibility/Registry")
-	if not self.__dict__.has_key("desktop"):
-	    self.desktop = self.registry.getDesktop(0)
+        if not self.__dict__.has_key("registry"):
+            self.registry = bonobo.get_object(
+                "OAFIID:Accessibility_Registry:1.0",
+                "Accessibility/Registry")
+        if not self.__dict__.has_key("desktop"):
+            self.desktop = self.registry.getDesktop(0)
 
     def start(self):
         """Starts event notification with the AT-SPI Registry.  This method
         only returns after 'stop' has been called.
         """
-	Accessible.init(self)
+        Accessible.init(self)
         bonobo.main()
 
     def stop(self):
@@ -137,9 +133,9 @@ class Registry:
         the AT-SPI Registry and then stops event notification with the
         AT-SPI Registry.
         """
-	Accessible.shutdown(self)
-	for listener in (self.__listeners + self.__keystrokeListeners):
-	    listener.deregister()
+        Accessible.shutdown(self)
+        for listener in (self.__listeners + self.__keystrokeListeners):
+            listener.deregister()
         bonobo.main_quit()
 
     def registerEventListener(self, callback, eventType):
@@ -149,7 +145,7 @@ class Registry:
         - callback: function to call with an AT-SPI event instance
         - eventType: string representing the type of event
         """
-       	listener = EventListener(self.registry, callback, eventType)
+        listener = EventListener(self.registry, callback, eventType)
         self.__listeners.append(listener)
 
     def deregisterEventListener(self, callback, eventType):
@@ -159,18 +155,18 @@ class Registry:
         - callback: function to call with an AT-SPI event instance
         - eventType: string representing the type of event
         """
-	found = True
-	while len(self.__listeners) and found:
-	    for i in range(0, len(self.__listeners)):
-	        if (self.__listeners[i].callback == callback) \
-	           and (self.__listeners[i].eventType == eventType):
+        found = True
+        while len(self.__listeners) and found:
+            for i in range(0, len(self.__listeners)):
+                if (self.__listeners[i].callback == callback) \
+                   and (self.__listeners[i].eventType == eventType):
                     # The __del__ method of the listener will unregister it.
                     #
-	            self.__listeners.pop(i)
-		    found = True
-		    break
-		else:
-		    found = False
+                    self.__listeners.pop(i)
+                    found = True
+                    break
+                else:
+                    found = False
 
     def registerKeystrokeListeners(self, callback):
         """Registers a single callback for all possible keystrokes.
@@ -210,10 +206,10 @@ class EventListener(Accessibility__POA.EventListener):
     def unref(self): pass
 
     def queryInterface(self, repo_id):
-	thiz = None
+        thiz = None
         if repo_id == "IDL:Accessibility/EventListener:1.0":
             thiz = self._this()
-	return thiz
+        return thiz
 
     def register(self):
         self._default_POA().the_POAManager.activate()
@@ -271,10 +267,10 @@ class KeystrokeListener(Accessibility__POA.DeviceEventListener):
     def unref(self): pass
 
     def queryInterface(self, repo_id):
-	thiz = None
+        thiz = None
         if repo_id == "IDL:Accessibility/EventListener:1.0":
             thiz = self._this()
-	return thiz
+        return thiz
 
     def register(self):
         d = self.registry.getDeviceEventController()
@@ -396,8 +392,8 @@ class Accessible:
 
         if Accessible._cache.has_key(e.source):
             obj = Accessible._cache[e.source]
-            if CACHE_VALUES:
-                obj.name = Event.getAnyData(e).value()
+            if obj.__dict__.has_key("name"):
+                del obj.name
             if obj.__dict__.has_key("label"):
                 del obj.label
 
@@ -413,8 +409,8 @@ class Accessible:
 
         if Accessible._cache.has_key(e.source):
             obj = Accessible._cache[e.source]
-            if CACHE_VALUES:
-                obj.description = Event.getAnyData(e).value()
+            if obj.__dict__.has_key("description"):
+                del obj.description
             if obj.__dict__.has_key("label"):
                 del obj.label
 
@@ -484,7 +480,7 @@ class Accessible:
 
         Returns a Python Accessible.
         """
-        
+
         obj = None
 
         if not acc:
@@ -499,12 +495,12 @@ class Accessible:
                 "         AT-SPI Accessible.\n"
                 "         Returning Python Accessible.")
             return acc
-            
-	# Comment these two lines out to eliminate any local caching 
-	# of Accessible objects.
-	#
+
+        # Comment these two lines out to eliminate any local caching
+        # of Accessible objects.
+        #
         #obj = Accessible(acc)
-	#return obj
+        #return obj
 
         if Accessible._cache.has_key(acc):
             obj = Accessible._cache[acc]
@@ -515,9 +511,9 @@ class Accessible:
         if not obj:
             obj = Accessible(acc)
 
-	if obj.valid:
+        if obj.valid:
             Accessible._cache[acc] = obj
-	    
+
         return obj
 
     makeAccessible = staticmethod(makeAccessible)
@@ -575,6 +571,14 @@ class Accessible:
         #
         try:
             self._acc = acc._narrow(Accessibility.Application)
+            try:
+                self.toolkitName = self._acc.toolkitName
+            except:
+                self.toolkitName = None
+            try:
+                self.version = self._acc.version
+            except:
+                self.version = None
         except:
             self._acc = acc._narrow(Accessibility.Accessible)
 
@@ -594,7 +598,7 @@ class Accessible:
                 self.valid = True
             except:
                 self.valid = False
-                
+
     def getStateString(self):
         """Returns a space-delimited string composed of the given object's
         Accessible state attribute.  This is for debug purposes.
@@ -700,7 +704,7 @@ class Accessible:
         """
 
         name = self._acc.name
-        if CACHE_VALUES:
+        if settings.cacheValues:
             self.name = name
         return name
 
@@ -709,7 +713,7 @@ class Accessible:
         """
 
         description = self._acc.description
-        if CACHE_VALUES:
+        if settings.cacheValues:
             self.description = self._acc.description
         return description
 
@@ -728,7 +732,7 @@ class Accessible:
             return None
         else:
             parent = Accessible.makeAccessible(obj);
-            if CACHE_VALUES:
+            if settings.cacheValues:
                 self.parent = parent
             return parent;
 
@@ -740,7 +744,7 @@ class Accessible:
 
         # We don't want to cache this value because it's possible that it
         # will continually change.
-        # if CACHE_VALUES:
+        # if settings.cacheValues:
         #     self.childCount = childCount
 
         return childCount
@@ -753,7 +757,7 @@ class Accessible:
 
         # We don't want to cache this value because it's possible that it
         # will continually change.
-        # if CACHE_VALUES:
+        # if settings.cacheValues:
         #     self.index = index
 
         return index
@@ -787,10 +791,23 @@ class Accessible:
             and (self.childCount > 0):
                 role = rolenames.ROLE_MENU
 
-        if CACHE_VALUES:
+        if settings.cacheValues:
             self.role = role
         return role
 
+    def __get_localized_rolename(self):
+        """Returns the Accessible role name of this object as a
+        localized string.  Most callers should use __get_role instead
+        since it returns a non-localized string that can be used for
+        comparison.
+        """
+
+        localizedRoleName = self._acc.getLocalizedRoleName()
+
+        if settings.cacheValues:
+            self.localizedRoleName = localizedRoleName
+        return localizedRoleName
+
     def __get_state(self):
         """Returns the Accessible StateSeq of this object, which is a
         sequence of Accessible StateTypes.
@@ -799,10 +816,10 @@ class Accessible:
         s = self._acc.getState()
         s = s._narrow(Accessibility.StateSet)
         state = s.getStates()
-	# [[[WDW - we don't seem to always get appropriate state changed
-	# information, so we will not cache state information.]]]
-	#
-        #if CACHE_VALUES:
+        # [[[WDW - we don't seem to always get appropriate state changed
+        # information, so we will not cache state information.]]]
+        #
+        #if settings.cacheValues:
         #    self.state = state
         return state
 
@@ -814,7 +831,7 @@ class Accessible:
         relations = []
         for relation in relationSet:
             relations.append(relation._narrow(Accessibility.Relation))
-        if CACHE_VALUES:
+        if settings.cacheValues:
             self.relations = relations
         return relations
 
@@ -830,20 +847,20 @@ class Accessible:
         #
         debug.println(debug.LEVEL_FINEST,
                       "Finding app for source.name=" \
-	              + self.accessibleNameToString())
+                      + self.accessibleNameToString())
         obj = self
         while obj.parent and (obj != obj.parent):
             obj = obj.parent
             debug.println(debug.LEVEL_FINEST,
                           "--> parent.name=" + obj.accessibleNameToString())
 
-	if (obj == obj.parent):
-	    debug.println(debug.LEVEL_SEVERE,
+        if (obj == obj.parent):
+            debug.println(debug.LEVEL_SEVERE,
                           "ERROR in Accessible.__get_app: obj == obj.parent!")
-	    return None
-	elif (obj.role != rolenames.ROLE_APPLICATION):
-	    debug.println(debug.LEVEL_FINEST,
-			  "ERROR in Accessible.__get_app: top most parent " \
+            return None
+        elif (obj.role != rolenames.ROLE_APPLICATION):
+            debug.println(debug.LEVEL_FINEST,
+                          "ERROR in Accessible.__get_app: top most parent " \
                           "(name='%s') is of role %s" % (obj.name, obj.role))
 
             # [[[TODO: We'll let this fall through for some cases.  It
@@ -858,7 +875,7 @@ class Accessible:
         debug.println(debug.LEVEL_FINEST, "Accessible app for %s is %s" \
                       % (self.accessibleNameToString(), \
                          obj.accessibleNameToString()))
-        if CACHE_VALUES:
+        if settings.cacheValues:
             self.app = obj
         return obj
 
@@ -890,7 +907,7 @@ class Accessible:
             # bug 319678.]]]
             #
             extents = component.getExtents(coordinateType)
-            #if CACHE_VALUES:
+            #if settings.cacheValues:
             #    self.extents = extents
             return extents
 
@@ -898,10 +915,10 @@ class Accessible:
         """Returns an object's label as a string.  The label is determined
         using the following logic:
 
-        1. If the object has a LABELLED_BY relation, return the text of
-           the targets of this relation
+        1. If the object has a name, return the name
 
-        2. Else if the object has no LABELLED_BY relation, return the name
+        2. Else if the object has a LABELLED_BY relation, return the text of
+           the targets of this relation
 
         3. Else if the object has no name, return the description
 
@@ -910,49 +927,45 @@ class Accessible:
 
         label = ""
 
+        # If the object has a name, just return the name.
+        #
+        if self.name and (len(self.name) > 0):
+            label = self.name
+
         # Does the object have a relation set?
         #
-        relations = self.relations
-        for relation in relations:
-            if relation.getRelationType() \
-                   == Accessibility.RELATION_LABELLED_BY:
-                target = Accessible.makeAccessible(relation.getTarget(0))
-                label = target.name
-                break
+        if (len(label) == 0):
+            relations = self.relations
+            for relation in relations:
+                if relation.getRelationType() \
+                       == Accessibility.RELATION_LABELLED_BY:
+                    target = Accessible.makeAccessible(relation.getTarget(0))
+                    label = target.name
+                    break
 
         # [[[WDW - HACK because push buttons can have labels as
-        # their children.  But, they can also be labelled by something.
+        # their children.  An example of the is is the Font: button on
+        # the General tab in the Editing Profile dialog in gnome-terminal.
+        # But, they can also be labelled by something.
         # So...we'll make the label be a combination of the thing
         # labelling them (above) plus their name or the combination of
         # the names of their children if the children exist.]]]
         #
         if self.role == rolenames.ROLE_PUSH_BUTTON:
-            if self.name and (len(self.name) > 0):
-                if len(label) > 0:
-                    label += " " + self.name
-                else:
-                    label = self.name
-            else:
-                for i in range(0, self.childCount):
-                    debug.println(debug.LEVEL_FINEST,
-                                  "Accessible.__get_label looking at child %d" % i)
-                    child = self.child(i)
-                    if child.role == rolenames.ROLE_LABEL:
-                        if child.name and (len(child.name) > 0):
-                            if len(label) > 0:
-                                label += " " + child.name
-                            else:
-                                label = child.name
-
-        # If the object doesn't have a relation, but has a name, return
-        # the name.
-        #
-        if (len(label) == 0) and self.name and (len(self.name) > 0):
-            label = self.name
+            for i in range(0, self.childCount):
+                debug.println(debug.LEVEL_FINEST,
+                              "Accessible.__get_label looking at child %d" % i)
+                child = self.child(i)
+                if child.role == rolenames.ROLE_LABEL:
+                    if child.name and (len(child.name) > 0):
+                        if len(label) > 0:
+                            label += " " + child.name
+                        else:
+                            label = child.name
 
         # If the object has no name, but has a description, return that
         #
-        elif (len(label) == 0) and self.description \
+        if (len(label) == 0) and self.description \
                  and (len(self.description) > 0):
             # [[[TODO: HACK because yelp actually goes through the trouble
             # of setting the description of some of its text areas to
@@ -961,7 +974,7 @@ class Accessible:
             if self.description != "no description":
                 label = self.description
 
-        if CACHE_VALUES:
+        if settings.cacheValues:
             self.label = label
         return label
 
@@ -974,7 +987,7 @@ class Accessible:
         action = self._acc.queryInterface("IDL:Accessibility/Action:1.0")
         if action:
             action = action._narrow(Accessibility.Action)
-        if CACHE_VALUES:
+        if settings.cacheValues:
             self.action = action
         return action
 
@@ -988,7 +1001,7 @@ class Accessible:
             "IDL:Accessibility/Component:1.0")
         if component:
             component = component._narrow(Accessibility.Component)
-        if CACHE_VALUES:
+        if settings.cacheValues:
             self.component = component
         return component
 
@@ -1002,7 +1015,7 @@ class Accessible:
             "IDL:Accessibility/Hypertext:1.0")
         if hypertext:
             hypertext = hypertext._narrow(Accessibility.Hypertext)
-        if CACHE_VALUES:
+        if settings.cacheValues:
             self.hypertext = hypertext
         return hypertext
 
@@ -1016,7 +1029,7 @@ class Accessible:
             "IDL:Accessibility/Image:1.0")
         if image:
             image = image._narrow(Accessibility.Image)
-        if CACHE_VALUES:
+        if settings.cacheValues:
             self.image = image
         return image
 
@@ -1030,7 +1043,7 @@ class Accessible:
             "IDL:Accessibility/Selection:1.0")
         if selection:
             selection = selection._narrow(Accessibility.Selection)
-        if CACHE_VALUES:
+        if settings.cacheValues:
             self.selection = selection
         return selection
 
@@ -1043,7 +1056,7 @@ class Accessible:
         table = self._acc.queryInterface("IDL:Accessibility/Table:1.0")
         if table:
             table = table._narrow(Accessibility.Table)
-        if CACHE_VALUES:
+        if settings.cacheValues:
             self.table = table
         return table
 
@@ -1056,7 +1069,7 @@ class Accessible:
         text = self._acc.queryInterface("IDL:Accessibility/Text:1.0")
         if text:
             text = text._narrow(Accessibility.Text)
-        if CACHE_VALUES:
+        if settings.cacheValues:
             self.text = text
         return text
 
@@ -1069,7 +1082,7 @@ class Accessible:
         value = self._acc.queryInterface("IDL:Accessibility/Value:1.0")
         if value:
             value = value._narrow(Accessibility.Value)
-        if CACHE_VALUES:
+        if settings.cacheValues:
             self.value = value
         return value
 
@@ -1100,6 +1113,8 @@ class Accessible:
             return self.__get_index()
         elif attr == "role":
             return self.__get_role()
+        elif attr == "localizedRoleName":
+            return self.__get_localized_rolename()
         elif attr == "state":
             return self.__get_state()
         elif attr == "relations":
@@ -1167,10 +1182,10 @@ class Accessible:
         """
 
         if includeApp:
-	    if self.app:
+            if self.app:
                 string = indent + "app.name=%-20s " \
                          % self.app.accessibleNameToString()
-	    else:
+            else:
                 string = indent + "app=None "
         else:
             string = indent
@@ -1358,19 +1373,19 @@ def getFrame(obj):
 
     debug.println(debug.LEVEL_FINEST,
                   "Finding frame for source.name="
-	          + obj.accessibleNameToString())
+                  + obj.accessibleNameToString())
 
     while obj \
           and (obj != obj.parent) \
           and (obj.role != rolenames.ROLE_FRAME):
         obj = obj.parent
         debug.println(debug.LEVEL_FINEST, "--> obj.name="
-		      + obj.accessibleNameToString())
+                      + obj.accessibleNameToString())
 
     if obj and (obj.role == rolenames.ROLE_FRAME):
-	pass
+        pass
     else:
-	obj = None
+        obj = None
 
     return obj
 
@@ -1489,6 +1504,29 @@ def getAcceleratorAndShortcut(obj):
 
     return [accelerator, fullShortcut]
 
+def getKnownApplications():
+    """Retrieves the list of currently running apps for the desktop
+    as a list of Accessible objects.
+    """
+
+    debug.println(debug.LEVEL_FINEST,
+                  "atspi.getKnownApplications...")
+
+    apps = []
+    registry = Registry()
+    for i in range(0, registry.desktop.childCount):
+        try:
+            acc = registry.desktop.getChildAtIndex(i)
+            app = Accessible.makeAccessible(acc)
+            if app:
+                apps.insert(0, app)
+        except:
+            debug.printException(debug.LEVEL_FINEST)
+
+    debug.println(debug.LEVEL_FINEST,
+                  "...orca._buildAppList")
+
+    return apps
 
 ########################################################################
 #                                                                      #
@@ -1498,19 +1536,19 @@ def getAcceleratorAndShortcut(obj):
 
 def printAncestry(child):
    """Prints a hierarchical view of a child's ancestry."""
-   
+
    if not child:
        return
-   
-   objects = [child]
+
+   ancestorList = [child]
    parent = child.parent
    while parent and (parent.parent != parent):
-      objects.insert(0, parent)
+      ancestorList.insert(0, parent)
       parent = parent.parent
 
    indent = ""
-   for object in objects:
-      print object.toString(indent + "+-", False)
+   for ancestor in ancestorList:
+      print ancestor.toString(indent + "+-", False)
       indent += "  "
 
 def printHierarchy(root, ooi, indent="", onlyShowing=True, omitManaged=True):
@@ -1531,10 +1569,10 @@ def printHierarchy(root, ooi, indent="", onlyShowing=True, omitManaged=True):
        print root.toString(indent + "(*)", False)
     else:
        print root.toString(indent + "+-", False)
-       
+
     rootManagesDescendants = root.state.count(\
         Accessibility.STATE_MANAGES_DESCENDANTS)
-    
+
     for i in range(0, root.childCount):
         child = root.child(i)
         if child == root:
@@ -1557,7 +1595,7 @@ def printHierarchy(root, ooi, indent="", onlyShowing=True, omitManaged=True):
                               indent + "  ",
                               onlyShowing,
                               omitManaged)
-      
+
 ########################################################################
 #                                                                      #
 # Testing functions.                                                   #
@@ -1567,14 +1605,14 @@ def printHierarchy(root, ooi, indent="", onlyShowing=True, omitManaged=True):
 def __printTopObject(child):
     parent = child
     while parent:
-	if not parent.parent:
+        if not parent.parent:
             print "RAW TOP:", parent.name, parent.role
         parent = parent.parent
     if (child.parent):
         accessible = Accessible.makeAccessible(child)
         app = accessible.app
         print "ACC TOP:", app.name, app.role
-	
+
 def __printDesktops():
     registry = Registry().registry
     print "There are %d desktops" % registry.getDesktopCount()
@@ -1585,15 +1623,15 @@ def __printDesktops():
         for j in range(0, desktop.childCount):
             app = desktop.getChildAtIndex(j)
             print "    App %d: name=%s role=%s" \
-		  % (j, app.name, app.role)
+                  % (j, app.name, app.role)
 
 def __notifyEvent(event):
         print event.type, event.source.name, \
               event.detail1, event.detail2,  \
               event.any_data
-	__printTopObject(event.source)
-	if not event.source.parent:
-	    print "NO PARENT:", event.source.name, event.source.role
+        __printTopObject(event.source)
+        if not event.source.parent:
+            print "NO PARENT:", event.source.name, event.source.role
 
 def __notifyKeystroke(event):
     print "keystroke type=%d hw_code=%d modifiers=%d event_string=(%s) " \
