@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2004-2005 Sun Microsystems Inc.
+# Copyright 2004-2006 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -59,7 +59,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         self._knownScripts = {}
         self._eventQueue   = Queue.Queue(0)
         self._gidle_id     = 0
-        
+
     ########################################################################
     #                                                                      #
     # METHODS FOR KEEPING TRACK OF LISTENERS REGISTERED WITH ATSPI         #
@@ -134,7 +134,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
 
         script = None
 
-        if settings.getSetting(settings.USE_CUSTOM_SCRIPTS, True):
+        if settings.enableCustomScripts:
             # Look for custom scripts first.
             #
             # We'll use the LEVEL_FINEST level for debug output here as
@@ -143,9 +143,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
             # a syntax error in it, so we want to give script writers
             # a vehicle for debugging these types of things.
             #
-            scriptPackages = settings.getSetting(
-                settings.SCRIPT_PACKAGES,
-                ["", "orca-scripts", "scripts"])
+            scriptPackages = settings.scriptPackages
 
             moduleName = settings.getScriptModuleName(app)
             module = None
@@ -183,6 +181,38 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
                     #
                     debug.printException(debug.LEVEL_SEVERE)
 
+        # If there is no custom script for an application, try seeing if
+        # there is a script for the toolkit of the application.  If there
+        # is, then try to use it.  If there isn't, then fall back to the
+        # default script. Note that this search is restricted to the "orca"
+        # package for now.
+        #
+        if (not script) \
+            and app \
+            and app.__dict__.has_key("toolkitName") \
+            and app.toolkitName:
+
+            try:
+                debug.println(
+                    debug.LEVEL_FINE,
+                    "Looking for toolkit script %s.py..." % app.toolkitName)
+                module = __import__(app.toolkitName,
+                                    globals(),
+                                    locals(),
+                                    [''])
+                script = module.Script(app)
+                debug.println(debug.LEVEL_FINE,
+                              "...found %s.py" % name)
+            except ImportError:
+                debug.println(
+                    debug.LEVEL_FINE,
+                    "...could not find %s.py" % app.toolkitName)
+            except:
+                debug.printException(debug.LEVEL_SEVERE)
+                debug.println(
+                    debug.LEVEL_SEVERE,
+                    "While attempting to import %s" % app.toolkitName)
+
         if not script:
             script = default.Script(app)
 
@@ -261,7 +291,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         #
         #self._processObjectEvent(atspi.Event(e))
         #return
-    
+
         # We ignore defunct objects and let the atspi module take care of them
         # for us.
         #
@@ -283,7 +313,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         if (e.type == "object:children-changed:remove") \
            and (e.source != self.registry.desktop):
             return
-           
+
         # We create the event here because it will ref everything we
         # want it to ref, thus allowing things to survive until they
         # are processed on the gidle thread.
@@ -304,7 +334,7 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         debug.println(debug.LEVEL_SEVERE,
                       "TIMEOUT: Looks like something has hung. Quitting Orca.")
         orca.shutdown()
-        
+
     def _dequeueObjectEvent(self):
         """Handles all events destined for scripts.  Called by the GTK
         idle thread.
@@ -319,21 +349,27 @@ class FocusTrackingPresenter(presentation_manager.PresentationManager):
         #
         #timer = threading.Timer(5.0, self._timeout)
         #timer.start()
+
+        debug.println(debug.eventDebugLevel,
+                      "\nvvvvv PROCESS OBJECT EVENT vvvvv")
         
         try:
             self._processObjectEvent(event)
         except:
             pass
         
+        debug.println(debug.eventDebugLevel,
+                      "^^^^^ PROCESS OBJECT EVENT ^^^^^\n")
+
         #timer.cancel()
         #del timer
-        
+
         if self._eventQueue.empty():
             self._gidle_id = 0
             return False
         else:
             return True
-        
+
     def _processObjectEvent(self, event):
         """Handles all events destined for scripts.
 
