@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2004-2005 Sun Microsystems Inc.
+# Copyright 2004-2006 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -20,8 +20,6 @@
 """Provides an experimental hierarchical navigation presentation
 manager for Orca.  This is mostly for exploratory purposes."""
 
-import sys
-
 import atspi
 import braille
 import debug
@@ -86,10 +84,10 @@ class HierarchicalPresenter(presentation_manager.PresentationManager):
     _SPECIALIZATION_LENGTH             = 11
 
     def __init__(self):
-    	self._navigationMode               = self._NAVIGATION_MODE_INTEROBJECT
-    	self._currentObject                = None
-    	self._currentObjectSpecializations = []
-    	self._currentSpecialization        = self._SPECIALIZATION_ACTION
+        self._navigationMode               = self._NAVIGATION_MODE_INTEROBJECT
+        self._currentObject                = None
+        self._currentObjectSpecializations = []
+        self._currentSpecialization        = self._SPECIALIZATION_ACTION
 
     ########################################################################
     #                                                                      #
@@ -102,11 +100,12 @@ class HierarchicalPresenter(presentation_manager.PresentationManager):
         accessible
         """
 
-	for i in range(0, len(orca.apps)):
-            if accessible.app == orca.apps[i]:
+        apps = atspi.getKnownApplications()
+        for i in range(0, len(apps)):
+            if accessible.app == apps[i]:
                 return i
 
-	return -1
+        return -1
 
     def _getIndentText(self, accessible):
         """Creates a string of numbers where each number represents a child
@@ -149,11 +148,12 @@ class HierarchicalPresenter(presentation_manager.PresentationManager):
     def _navigateInterObject(self, keystring):
         """Navigates between objects in the component hierarchy."""
 
+        apps = atspi.getKnownApplications()
         if keystring == "Up":
             if self._currentObject == self._currentObject.app:
                 index = max(self._getApplicationIndex(self._currentObject) - 1,
                             0)
-                self._currentObject = orca.apps[index]
+                self._currentObject = apps[index]
                 self._displayAccessible(self._currentObject)
             else:
                 parent = self._currentObject.parent
@@ -163,13 +163,13 @@ class HierarchicalPresenter(presentation_manager.PresentationManager):
         elif keystring == "Down":
             if self._currentObject == self._currentObject.app:
                 index = min(self._getApplicationIndex(self._currentObject) + 1,
-                            len(orca.apps) - 1)
-                self._currentObject = orca.apps[index]
+                            len(apps) - 1)
+                self._currentObject = apps[index]
                 self._displayAccessible(self._currentObject)
             else:
                 parent = self._currentObject.parent
                 index = min(self._currentObject.index + 1,
-			    parent.childCount - 1)
+                            parent.childCount - 1)
                 self._currentObject = parent.child(index)
                 self._displayAccessible(self._currentObject)
         elif keystring == "Left":
@@ -262,7 +262,7 @@ class HierarchicalPresenter(presentation_manager.PresentationManager):
             return
 
         brltext = "Selection: %d selected children" \
-		  % selection.nSelectedChildren
+                  % selection.nSelectedChildren
 
         debug.println(debug.LEVEL_OFF, brltext)
 
@@ -429,6 +429,7 @@ class HierarchicalPresenter(presentation_manager.PresentationManager):
 
         speech.speak(_("Switching to hierarchical navigation mode."))
 
+        apps = atspi.getKnownApplications()
 
         win = orca.findActiveWindow()
 
@@ -436,9 +437,9 @@ class HierarchicalPresenter(presentation_manager.PresentationManager):
             self._currentObject = win.app
             self._displayAccessible(self._currentObject)
         else:
-            self._currentObject = orca.apps[0]
+            self._currentObject = apps[0]
             self._displayAccessible(self._currentObject)
-            
+
     def deactivate(self):
         """Called when this presentation manager is deactivated."""
         orca.outlineAccessible(None)
