@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2004-2005 Sun Microsystems Inc.
+# Copyright 2004-2006 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -19,6 +19,7 @@
 
 import gtk
 import os, signal, sys
+import string
 import time
 
 import atspi
@@ -26,11 +27,11 @@ import braille
 import debug
 import keynames
 import keybindings
-#import mag - [[[TODO: WDW - disable until I can figure out how to
-#             resolve the GNOME reference in mag.py.  This is logged
-#             as bugzilla bug 319643.]]]
+import mag
+import rolenames
 import settings
 import speech
+import threading
 
 from input_event import BrailleEvent
 from input_event import KeyboardEvent
@@ -38,13 +39,20 @@ from input_event import InputEventHandler
 
 from orca_i18n import _           # for gettext support
 
+# The user-settings module (see _loadUserSettings).
+#
+_userSettings = None
+
+# set each time a keyboard or braille event is received.
+#
+
 # The InputEvent instance representing the last input event.  This is
 # set each time a keyboard or braille event is received.
 #
 lastInputEvent = None
 
-# A new modifier to use (currently bound to the "Insert" key) to represent
-# special Orca key sequences.
+# A new modifier to use (set by the press of any key in the
+# settings.orcaModifierKeys list) to represent the Orca modifier.
 #
 MODIFIER_ORCA = 8
 
@@ -108,11 +116,6 @@ def _switchToNextPresentationManager(script=None, inputEvent=None):
 #                                                                      #
 ########################################################################
 
-# List of all the running apps we know about.  Each element is a Python
-# Accessible instance.
-#
-apps = []
-
 # The Accessible that has visual focus.
 #
 locusOfFocus = None
@@ -209,39 +212,16 @@ def findActiveWindow():
     """
 
     window = None
+    apps = atspi.getKnownApplications()
     for app in apps:
         for i in range(0, app.childCount):
             state = app.child(i).state
             if state.count(atspi.Accessibility.STATE_ACTIVE) > 0:
                 window = app.child(i)
-		break
+                break
 
     return window
 
-def _buildAppList(registry):
-    """Retrieves the list of currently running apps for the desktop and
-    populates the apps list attribute with these apps.
-    """
-
-    global apps
-
-    debug.println(debug.LEVEL_FINEST,
-                  "orca._buildAppList...")
-
-    apps = []
-
-    for i in range(0, registry.desktop.childCount):
-        acc = registry.desktop.getChildAtIndex(i)
-        try:
-            app = atspi.Accessible.makeAccessible(acc)
-            if app:
-                apps.insert(0, app)
-        except:
-            debug.printException(debug.LEVEL_SEVERE)
-
-    debug.println(debug.LEVEL_FINEST,
-                  "...orca._buildAppList")
-
 def _onChildrenChanged(e):
     """Tracks children-changed events on the desktop to determine when
     apps start and stop.
@@ -265,12 +245,6 @@ def _onChildrenChanged(e):
             shutdown()
             return
 
-        # [[[TODO: WDW - Note the call to _buildAppList - that will update the
-        # apps[] list.  If this logic is changed in the future, the apps list
-        # will most likely needed to be updated here.]]]
-        #
-        _buildAppList(registry)
-
 ########################################################################
 #                                                                      #
 # Keyboard Event Recording Support                                     #
@@ -385,8 +359,8 @@ def cycleDebugLevel(script=None, inputEvent=None):
 def cycleDebugLevel(script=None, inputEvent=None):
     global _debugLevel
 
-    level = debug.getDebugLevel()
-    
+    level = debug.debugLevel
+
     if level == debug.LEVEL_ALL:
         level = debug.LEVEL_FINEST
     elif level == debug.LEVEL_FINEST:
@@ -406,8 +380,8 @@ def cycleDebugLevel(script=None, inputEvent=None):
     elif level == debug.LEVEL_OFF:
         level = debug.LEVEL_ALL
 
-    debug.setDebugLevel(level)
-    
+    debug.debugLevel = level
+
     if level == debug.LEVEL_ALL:
         speech.speak(_("Debug level all."))
     elif level == debug.LEVEL_FINEST:
@@ -440,6 +414,7 @@ def printApps(script=None, inputEvent=None):
 
     level = debug.LEVEL_OFF
 
+    apps = atspi.getKnownApplications()
     debug.println(level, "There are %d Accessible applications" % len(apps))
     for app in apps:
         debug.printDetails(level, "  App: ", app, False)
@@ -472,7 +447,6 @@ def printActiveApp(script=None, inputEvent=None):
             debug.println(level, "Active application: None")
         else:
             debug.println(level, "Active application: %s" % app.name)
-            printAccessibleTree(level, "  ", findActiveWindow())
 
     return True
 
@@ -504,7 +478,7 @@ def enterLearnMode(script=None, inputEvent=None):
         _("Entering learn mode.  Press any key to hear its function. " \
           + "To exit learn mode, press the escape key."))
     braille.displayMessage(_("Learn mode.  Press escape to exit."))
-    settings.setLearnModeEnabled(True)
+    settings.learnModeEnabled = True
     return True
 
 def exitLearnMode(script=None, inputEvent=None):
@@ -515,7 +489,7 @@ def exitLearnMode(script=None, inputEvent=None):
 
     speech.speak(_("Exiting learn mode."))
     braille.displayMessage(_("Exiting learn mode."))
-    settings.setLearnModeEnabled(False)
+    settings.learnModeEnabled = False
     return True
 
 ########################################################################
@@ -531,37 +505,178 @@ def exitLearnMode(script=None, inputEvent=None):
 
 # Keybindings that Orca itself cares about.
 #
-_keybindings = None
+_keyBindings = None
 
-# True if the insert key is currently pressed.  We will use the insert
-# key as a modifier for Orca, and it will be presented as the "insert"
-# modifier string.
+# True if the orca modifier key is currently pressed.
 #
-_insertPressed = False
+_orcaModifierPressed = False
+
+def _isPrintableKey(event_string):
+    """Return an indication of whether this is an alphanumeric or
+       punctuation key.
+
+    Arguments:
+    - event: the event string
+
+    Returns True if this is an alphanumeric or punctuation key.
+    """
+
+    reply = event_string in string.printable
+    debug.println(debug.LEVEL_FINEST,
+                  "orca._echoPrintableKey: returning: %s" % reply)
+    return reply
+
+def _isModifierKey(event_string):
+    """Return an indication of whether this is a modifier key.
+
+    Arguments:
+    - event: the event string
+
+    Returns True if this is a modifier key
+    """
+
+    # [[[TODO:richb - the Fn key on my laptop doesn't seem to generate an
+    #    event.]]]
+
+    modifierKeys = [ 'Alt_L', 'Alt_R', 'Control_L', 'Control_R', \
+                     'Shift_L', 'Shift_R', 'Meta_L', 'Meta_R', "Insert" ]
+
+    reply = event_string in modifierKeys
+    debug.println(debug.LEVEL_FINEST,
+                  "orca._echoModifierKey: returning: %s" % reply)
+    return reply
+
+def _isLockingKey(event_string):
+    """Return an indication of whether this is a locking key.
+
+    Arguments:
+    - event: the event string
+
+    Returns True if this is a locking key.
+    """
+
+    lockingKeys = [ "Caps_Lock", "Num_Lock", "Scroll_Lock" ]
+
+    reply = event_string in lockingKeys
+    debug.println(debug.LEVEL_FINEST,
+                  "orca._echoLockingKey: returning: %s" % reply)
+    return reply
+
+def _isFunctionKey(event_string):
+    """Return an indication of whether this is a function key.
+
+    Arguments:
+    - event: the event string
+
+    Returns True if this is a function key.
+    """
+
+    # [[[TODO:richb - what should be done about the function keys on the left
+    #    side of my Sun keyboard and the other keys (like Scroll Lock), which
+    #    generate "Fn" key events?]]]
+
+    functionKeys = [ "F1", "F2", "F3", "F4", "F5", "F6",
+                     "F7", "F8", "F9", "F10", "F11", "F12" ]
+
+    reply = event_string in functionKeys
+    debug.println(debug.LEVEL_FINEST,
+                  "orca._echoFunctionKey: returning: %s" % reply)
+    return reply
+
+def _isActionKey(event_string):
+    """Return an indication of whether this is an action key.
+
+    Arguments:
+    - event: the event string
+
+    Returns True if this is an action key.
+    """
+
+    actionKeys = [ "space", "Return", "Escape", "Tab", "BackSpace", "Delete",
+                   "Left", "Right", "Up", "Down", "Page_Up", "Page_Down" ]
+
+    reply = event_string in actionKeys
+    debug.println(debug.LEVEL_FINEST,
+                  "orca._echoActionKey: returning: %s" % reply)
+    return reply
+
+def _keyEcho(event):
+    """If the keyEcho setting is enabled, check to see what type of key
+    event it is and echo it via speech, if the user wants that type of
+    key echoed.
 
-def _keyEcho(key):
-    """If the keyEcho setting is enabled, echoes the key via speech.
     Uppercase keys will be spoken using the "uppercase" voice style,
     whereas lowercase keys will be spoken using the "default" voice style.
 
     Arguments:
-    - key: a string representing the key name to echo.
+    - event: an AT-SPI DeviceEvent
     """
 
-    if not settings.getSetting(settings.USE_KEY_ECHO, False):
-        return
-    if key.isupper():
-        voices = settings.getSetting(settings.VOICES, None)
-        speech.speak(key, voices[settings.UPPERCASE_VOICE])
-    else:
+    event_string = event.event_string
+    debug.println(debug.LEVEL_FINEST,
+                  "orca._keyEcho: string to echo: %s" % event_string)
+
+    voices = settings.voices
+    voice = voices[settings.DEFAULT_VOICE]
+
+    # If key echo is enabled, then check to see what type of key event
+    # it is and echo it via speech, if the user wants that type of key
+    # echoed.
+    #
+    if settings.enableKeyEcho:
+        if _isPrintableKey(event_string):
+            if not settings.enablePrintableKeys:
+                return
+
+            if event_string.isupper():
+                voice = voices[settings.UPPERCASE_VOICE]
+
+        elif _isModifierKey(event_string):
+            if not settings.enableModifierKeys:
+                return
+
+            # The control characters come through as control characters,
+            # so we just turn them into their ASCII equivalent.  NOTE that
+            # the upper case ASCII characters will be used (e.g., ctrl+a
+            # will be turned into the string "A").  All these checks here
+            # are to just do some sanity checking before doing the
+            # conversion. [[[WDW - this is making assumptions about
+            # mapping ASCII control characters to to UTF-8.]]]
+            #
+            if (event.modifiers & (1 << atspi.Accessibility.MODIFIER_CONTROL)) \
+               and (not event.is_text) and (len(event_string) == 1):
+                value = ord(event.event_string[0])
+                if value < 32:
+                    event_string = chr(value + 0x40)
+
+        elif _isLockingKey(event_string):
+            if not settings.enableLockingKeys:
+                return
+
+        elif _isFunctionKey(event_string):
+            if not settings.enableFunctionKeys:
+                return
+
+        elif _isActionKey(event_string):
+            if not settings.enableActionKeys:
+                return
+
+        else:
+            debug.println(debug.LEVEL_FINEST,
+                  "orca._keyEcho: event string not handled: %s" % event_string)
+            return
+
         # Check to see if there are localized words to be spoken for
         # this key event.
         try:
-            key = keynames.keynames[key]
+            event_string = keynames.keynames[event_string]
         except:
             debug.printException(debug.LEVEL_FINEST)
             pass
-        speech.speak(key)
+
+        debug.println(debug.LEVEL_FINEST,
+                      "orca._keyEcho: speaking: %s" % event_string)
+        speech.speak(event_string, voice)
 
 def _processKeyboardEvent(event):
     """The primary key event handler for Orca.  Keeps track of various
@@ -578,7 +693,7 @@ def _processKeyboardEvent(event):
     """
 
     global lastInputEvent
-    global _insertPressed
+    global _orcaModifierPressed
 
     event_string = event.event_string
 
@@ -590,37 +705,25 @@ def _processKeyboardEvent(event):
         _keystrokesFile.write(string + "\n")
     debug.printInputEvent(debug.LEVEL_FINE, string)
 
+    orcaModifierKeys = settings.orcaModifierKeys
+
     if event.type == atspi.Accessibility.KEY_PRESSED_EVENT:
         # Key presses always interrupt speech.
         #
         speech.stop()
 
-        # The control characters come through as control characters,
-        # so we just turn them into their ASCII equivalent.  NOTE that
-        # the upper case ASCII characters will be used (e.g., ctrl+a
-        # will be turned into the string "A").  All these checks here
-        # are to just do some sanity checking before doing the
-        # conversion. [[[WDW - this is making assumptions about
-        # mapping ASCII control characters to to UTF-8.]]]
-        #
-        if (event.modifiers & (1 << atspi.Accessibility.MODIFIER_CONTROL)) \
-           and (not event.is_text) and (len(event.event_string) == 1):
-            value = ord(event.event_string[0])
-            if value < 32:
-                event_string = chr(value + 0x40)
-
-        _keyEcho(event_string)
+        _keyEcho(event)
 
         # We treat the Insert key as a modifier - so just swallow it and
         # set our internal state.
         #
-        if (event_string == "Insert") or (event_string == "KP_Insert"):
-            _insertPressed = True
+        if orcaModifierKeys.count(event_string):
+            _orcaModifierPressed = True
             return True
 
-    elif event.type == atspi.Accessibility.KEY_RELEASED_EVENT \
-         and ((event_string == "Insert") or (event_string == "KP_Insert")):
-        _insertPressed = False
+    elif (event.type == atspi.Accessibility.KEY_RELEASED_EVENT) \
+         and orcaModifierKeys.count(event_string):
+        _orcaModifierPressed = False
         return True
 
     # Orca gets first stab at the event.  Then, the presenter gets
@@ -629,17 +732,16 @@ def _processKeyboardEvent(event):
     # scripts to override fundamental Orca key bindings.]]]
     #
     keyboardEvent = KeyboardEvent(event)
-    if _insertPressed:
+    if _orcaModifierPressed:
         keyboardEvent.modifiers |= (1 << MODIFIER_ORCA)
 
     consumed = False
     try:
-        consumed = _keybindings.consumeKeyboardEvent(None, keyboardEvent)
+        consumed = _keyBindings.consumeKeyboardEvent(None, keyboardEvent)
         if (not consumed) and (_currentPresentationManager >= 0):
             consumed = _PRESENTATION_MANAGERS[_currentPresentationManager].\
                        processKeyboardEvent(keyboardEvent)
-        if (not consumed) and settings.getSetting(settings.LEARN_MODE_ENABLED,
-                                                  False):
+        if (not consumed) and settings.learnModeEnabled:
             if event.type == atspi.Accessibility.KEY_PRESSED_EVENT:
                 braille.displayMessage(event_string)
                 speech.speak(event_string)
@@ -688,8 +790,7 @@ def _processBrailleEvent(command):
     except:
         debug.printException(debug.LEVEL_SEVERE)
 
-    if (not consumed) and settings.getSetting(settings.LEARN_MODE_ENABLED,
-                                              False):
+    if (not consumed) and settings.learnModeEnabled:
         consumed = True
 
     return consumed
@@ -778,6 +879,70 @@ def outlineAccessible(accessible, erasePrevious=True):
 #                                                                      #
 ########################################################################
 
+def _loadUserSettings(script=None, inputEvent=None):
+    """Loads (and reloads) the user settings module, reinitializing
+    things such as speech if necessary.
+
+    Returns True to indicate the input event has been consumed.
+    """
+
+    global _userSettings
+
+    # Shutdown the output drivers and give them a chance to die.
+    speech.shutdown()
+    braille.shutdown()
+    mag.shutdown()
+    time.sleep(1)
+
+    reloaded = False
+    if _userSettings:
+        try:
+            reload(_userSettings)
+            reloaded = True
+        except ImportError:
+            debug.printException(debug.LEVEL_SEVERE)
+    else:
+        try:
+            _userSettings = __import__("user-settings")
+        except ImportError:
+            debug.printException(debug.LEVEL_SEVERE)
+
+    if settings.enableSpeech:
+        try:
+            speech.init()
+            if reloaded:
+                speech.speak(_("Orca user settings reloaded."))
+            debug.println(debug.LEVEL_CONFIGURATION,
+                          "Speech module has been initialized.")
+        except:
+            debug.println(debug.LEVEL_SEVERE,
+                          "Could not initialize connection to speech.")
+    else:
+        debug.println(debug.LEVEL_CONFIGURATION,
+                      "Speech module has NOT been initialized.")
+
+    if settings.enableBraille:
+        try:
+            braille.init(_processBrailleEvent, 7)
+        except:
+            debug.println(debug.LEVEL_WARNING,
+                          "Could not initialize connection to braille.")
+
+    if settings.enableMagnifier:
+        try:
+            mag.init()
+            debug.println(debug.LEVEL_CONFIGURATION,
+                          "Magnification module has been initialized.")
+        except:
+            debug.printException(debug.LEVEL_SEVERE)
+            debug.println(debug.LEVEL_SEVERE,
+                          "Could not initialize connection to magnifier.")
+    else:
+        debug.println(debug.LEVEL_CONFIGURATION,
+                      "Magnification module has NOT been initialized.")
+
+    return True
+
 # If True, this module has been initialized.
 #
 _initialized = False
@@ -792,19 +957,17 @@ def init(registry):
     """
 
     global _initialized
-    global _keybindings
+    global _keyBindings
 
     if _initialized:
         return False
 
-    registry.registerKeystrokeListeners(_processKeyboardEvent)
-
-    _keybindings = keybindings.KeyBindings()
+    _keyBindings = keybindings.KeyBindings()
 
     enterLearnModeHandler = InputEventHandler(\
         enterLearnMode,
         _("Enters learn mode.  Press escape to exit learn mode."))
-    _keybindings.add(keybindings.KeyBinding("F1", \
+    _keyBindings.add(keybindings.KeyBinding("F1", \
                                             1 << MODIFIER_ORCA, \
                                             1 << MODIFIER_ORCA, \
                                             enterLearnModeHandler))
@@ -812,7 +975,7 @@ def init(registry):
     decreaseSpeechRateHandler = InputEventHandler(\
         speech.decreaseSpeechRate,
         _("Decreases the speech rate."))
-    _keybindings.add(keybindings.KeyBinding("Left", \
+    _keyBindings.add(keybindings.KeyBinding("Left", \
                                             1 << MODIFIER_ORCA, \
                                             1 << MODIFIER_ORCA,
                                             decreaseSpeechRateHandler))
@@ -820,17 +983,17 @@ def init(registry):
     increaseSpeechRateHandler = InputEventHandler(\
         speech.increaseSpeechRate,
         _("Increases the speech rate."))
-    _keybindings.add(keybindings.KeyBinding("Right", \
+    _keyBindings.add(keybindings.KeyBinding("Right", \
                                             1 << MODIFIER_ORCA, \
                                             1 << MODIFIER_ORCA,
                                             increaseSpeechRateHandler))
 
     shutdownHandler = InputEventHandler(shutdown, _("Quits Orca"))
-    _keybindings.add(keybindings.KeyBinding("F12", \
+    _keyBindings.add(keybindings.KeyBinding("F12", \
                                             0, \
                                             0,
                                             shutdownHandler))
-    _keybindings.add(keybindings.KeyBinding("SunF37", \
+    _keyBindings.add(keybindings.KeyBinding("SunF37", \
                                             0, \
                                             0,
                                             shutdownHandler))
@@ -838,41 +1001,42 @@ def init(registry):
     keystrokeRecordingHandler = InputEventHandler(\
         toggleKeystrokeRecording,
         _("Toggles keystroke recording on and off."))
-    _keybindings.add(keybindings.KeyBinding("Pause", \
+    _keyBindings.add(keybindings.KeyBinding("Pause", \
                                             0, \
                                             0,
                                             keystrokeRecordingHandler))
 
-    resetSpeechHandler = InputEventHandler(\
-        resetSpeech,
-        _("Forces a reset of the speech service."))
-    _keybindings.add(keybindings.KeyBinding("s", \
-                                            1 << MODIFIER_ORCA, \
-                                            1 << MODIFIER_ORCA, \
-                                            resetSpeechHandler))
+    loadUserSettingsHandler = InputEventHandler(\
+        _loadUserSettings,
+        _("Reloads user settings and reinitializes services as necessary."))
+    _keyBindings.add(keybindings.KeyBinding(
+        "s", \
+        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
+        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
+        loadUserSettingsHandler))
 
     listAppsHandler = InputEventHandler(
         printApps,
         _("Prints a debug listing of all known applications to the console where Orca is running."))
-    _keybindings.add(keybindings.KeyBinding(
-        "F5", 
-        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL), 
+    _keyBindings.add(keybindings.KeyBinding(
+        "F5",
+        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
         1 << MODIFIER_ORCA,
         listAppsHandler))
 
     cycleDebugLevelHandler = InputEventHandler(
         cycleDebugLevel,
         _("Cycles the debug level at run time."))
-    _keybindings.add(keybindings.KeyBinding(
-        "F5", 
-        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL), 
-        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL), 
+    _keyBindings.add(keybindings.KeyBinding(
+        "F5",
+        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
+        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
         cycleDebugLevelHandler))
 
     printActiveAppHandler = InputEventHandler(\
         printActiveApp,
         _("Prints debug information about the currently active application to the console where Orca is running."))
-    _keybindings.add(keybindings.KeyBinding("F6", \
+    _keyBindings.add(keybindings.KeyBinding("F6", \
                                             1 << MODIFIER_ORCA, \
                                             1 << MODIFIER_ORCA,
                                             printActiveAppHandler))
@@ -880,87 +1044,48 @@ def init(registry):
     printAncestryHandler = InputEventHandler(\
         printAncestry,
         _("Prints debug information about the ancestry of the object with focus"))
-    _keybindings.add(keybindings.KeyBinding(
+    _keyBindings.add(keybindings.KeyBinding(
         "F7",
-        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL), 
+        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
         1 << MODIFIER_ORCA,
         printAncestryHandler))
 
     printHierarchyHandler = InputEventHandler(\
         printHierarchy,
         _("Prints debug information about the application with focus"))
-    
-    _keybindings.add(keybindings.KeyBinding(
+
+    _keyBindings.add(keybindings.KeyBinding(
         "F7",
-        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL), 
-        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL), 
+        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
+        (1 << MODIFIER_ORCA | 1 << atspi.Accessibility.MODIFIER_CONTROL),
         printHierarchyHandler))
 
     nextPresentationManagerHandler = InputEventHandler(\
         _switchToNextPresentationManager,
         _("Switches to the next presentation manager."))
-    _keybindings.add(keybindings.KeyBinding("F8", \
+    _keyBindings.add(keybindings.KeyBinding("F8", \
                                             1 << MODIFIER_ORCA, \
                                             1 << MODIFIER_ORCA,
                                             nextPresentationManagerHandler))
 
-    if settings.getSetting(settings.USE_SPEECH, True):
-        try:
-            speech.init()
-            debug.println(debug.LEVEL_CONFIGURATION,
-                          "Speech module has been initialized.")
-        except:
-            debug.println(debug.LEVEL_SEVERE,
-                          "Could not initialize connection to speech.")
-    else:
-        debug.println(debug.LEVEL_CONFIGURATION,
-                      "Speech module has NOT been initialized.")
-
-    if settings.getSetting(settings.USE_BRAILLE, False):
-        try:
-            braille.init(_processBrailleEvent, 7)
-        except:
-            debug.println(debug.LEVEL_SEVERE,
-                          "Could not initialize connection to braille.")
-
-    if settings.getSetting(settings.USE_MAGNIFIER, False):
-        try:
-            mag.init()
-            debug.println(debug.LEVEL_CONFIGURATION,
-                          "Magnification module has been initialized.")
-        except:
-            debug.println(debug.LEVEL_SEVERE,
-                          "Could not initialize connection to magnifier.")
-    else:
-        debug.println(debug.LEVEL_CONFIGURATION,
-                      "Magnification module has NOT been initialized.")
-
-    # Build list of accessible apps.
-    #
-    _buildAppList(registry)
-
     # Create and load an app's script when it is added to the desktop
     #
     registry.registerEventListener(_onChildrenChanged,
-				   "object:children-changed:")
+                                   "object:children-changed:")
+
+    _loadUserSettings()
+
+    registry.registerKeystrokeListeners(_processKeyboardEvent)
 
     _initialized = True
     return True
 
-def resetSpeech(script=None, inputEvent=None):
-    """Kicks the speech server in the pants and re-establishes a
-    connection to it."""
-    speech.reset()
-    time.sleep(1) # give the driver a chance to die.
-    speech.reset("Speech has been reset.")    
-    return True
-
 def start(registry):
     """Starts Orca.
     """
 
     global _PRESENTATION_MANAGERS
-    
+
     if not _initialized:
         init(registry)
 
@@ -971,16 +1096,33 @@ def start(registry):
         debug.printException(debug.LEVEL_SEVERE)
 
     if not _PRESENTATION_MANAGERS:
+
+        # [[[WDW - comment out hierarchical_presenter for now.  It
+        # relies on the list of known applications, and we've disabled
+        # that due to a hang in a call to getChildAtIndex in
+        # atspi.getKnownApplications.]]]
+        #
+        #import focus_tracking_presenter
+        #import hierarchical_presenter
+        #_PRESENTATION_MANAGERS = \
+        #    [focus_tracking_presenter.FocusTrackingPresenter(),
+        #     hierarchical_presenter.HierarchicalPresenter()]
+
         import focus_tracking_presenter
-        import hierarchical_presenter
         _PRESENTATION_MANAGERS = \
-            [focus_tracking_presenter.FocusTrackingPresenter(),
-             hierarchical_presenter.HierarchicalPresenter()]
-    
+            [focus_tracking_presenter.FocusTrackingPresenter()]
+
     _switchToPresentationManager(0) # focus_tracking_presenter
 
     registry.start()
 
+def abort():
+    sys.exit(1)
+
+def timeout():
+    print "TIMEOUT: Hung while trying to shutdown.  Aborting."
+    abort()
+
 def shutdown(script=None, inputEvent=None):
     """Exits Orca.  Unregisters any event listeners and cleans up.  Also
     quits the bonobo main loop and resets the initialized state to False.
@@ -994,6 +1136,13 @@ def shutdown(script=None, inputEvent=None):
     if not _initialized:
         return False
 
+    # [[[TODO: WDW - the timer stuff is an experiment to see if
+    # we can recover from hangs.  It's only experimental, so it's
+    # commented out for now.]]]
+    #
+    #timer = threading.Timer(5.0, timeout)
+    #timer.start()
+
     speech.speak(_("goodbye."))
     braille.displayMessage(_("Goodbye."))
 
@@ -1001,47 +1150,85 @@ def shutdown(script=None, inputEvent=None):
     #
     registry = atspi.Registry()
     registry.deregisterEventListener(_onChildrenChanged,
-            	                     "object:children-changed:")
+                                     "object:children-changed:")
 
     if _currentPresentationManager >= 0:
         _PRESENTATION_MANAGERS[_currentPresentationManager].deactivate()
 
     # Shutdown all the other support.
     #
-    if settings.getSetting(settings.USE_SPEECH, True):
+    if settings.enableSpeech:
         speech.shutdown()
-    if settings.getSetting(settings.USE_BRAILLE, False):
+    if settings.enableBraille:
         braille.shutdown();
-    if settings.getSetting(settings.USE_MAGNIFIER, False):
+    if settings.enableMagnifier:
         mag.shutdown();
 
     registry.stop()
 
+    #timer.cancel()
+    #del timer
+
     _initialized = False
     return True
 
-def shutdownAndExit(signum, frame):
-    print "Goodbye."
-    try:
-	shutdown()
-    except:
-        pass
-    sys.exit()
+exitCount = 0
+def shutdownOnSignal(signum, frame):
+    global exitCount
 
-def fastExit(signum, frame):
-    print "Goodbye."
-    sys.exit()
+    print "Shutting down and exiting due to signal =", signum
+
+    # Well...we'll try to exit nicely, but if we keep getting called,
+    # something bad is happening, so just quit.
+    #
+    if exitCount:
+        abort()
+    else:
+        exitCount += 1
+
+    # Try to do a graceful shutdown if we can.
+    #
+    try:
+        shutdown()
+        return
+    except:
+        abort()
+
+def abortOnSignal(signum, frame):
+    print "Aborting due to signal =", signum
+    abort()
 
 def main():
     userprefs = os.path.join(os.environ["HOME"], ".orca")
     sys.path.insert(0, userprefs)
     sys.path.insert(0, '') # current directory
-    signal.signal(signal.SIGINT, shutdownAndExit)
-    signal.signal(signal.SIGQUIT, shutdownAndExit)
-    signal.signal(signal.SIGSEGV, fastExit)
+
+    signal.signal(signal.SIGHUP, shutdownOnSignal)
+    signal.signal(signal.SIGINT, shutdownOnSignal)
+    signal.signal(signal.SIGTERM, shutdownOnSignal)
+    signal.signal(signal.SIGQUIT, shutdownOnSignal)
+    signal.signal(signal.SIGSEGV, abortOnSignal)
+
     registry = atspi.Registry()
     init(registry)
+
+    # Do not run Orca if accessibility has not been enabled.
+    #
+    import commands
+    a11yEnabled = commands.getoutput(\
+        "gconftool-2 --get /desktop/gnome/interface/accessibility")
+    if a11yEnabled != "true":
+        message = _("Accessibility has not been enabled for this session.")
+        print message
+        speech.speak(message)
+        message = _("Please run orca-setup and then logout and log back in.")
+        print message
+        speech.speak(message)
+        braille.displayMessage(message)
+        abort()
+
     start(registry)
+    return 0
 
 if __name__ == "__main__":
-    main()
+    sys.exit(main())
