@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005 Sun Microsystems Inc.
+# Copyright 2005-2006 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -78,28 +78,6 @@ class Script(default.Script):
                                _("CalendarEvent"),
                                _("calendar event"))
 
-
-    def walkComponentHierarchy(self, obj):
-        """Debug routine to print out the hierarchy of components for the
-           given object.
-
-        Arguments:
-        - obj: the component to start from
-        """
-
-        print "<<<<---- Component Hierachy ---->>>>"
-        print "START: Obj:", obj.name, obj.role
-        parent = obj
-        while parent:
-            if parent != obj:
-                if not parent.parent:
-                    print "TOP: Parent:", parent.name, parent.role
-                else:
-                    print "Parent:", parent.name, parent.role
-            parent = parent.parent
-        print "<<<<============================>>>>"
-
-
     def readPageTab(self, tab):
         """Speak/Braille the given page tab. The speech verbosity is set
            to VERBOSITY_LEVEL_BRIEF for this operation and then restored
@@ -112,28 +90,23 @@ class Script(default.Script):
         brailleGen = self.brailleGenerator
         speechGen = self.speechGenerator
 
-        savedSpeechVerbosityLevel = \
-            settings.getSetting(settings.SPEECH_VERBOSITY_LEVEL)
+        savedSpeechVerbosityLevel = settings.speechVerbosityLevel
         settings.speechVerbosityLevel = settings.VERBOSITY_LEVEL_BRIEF
         utterances = speechGen.getSpeech(tab, False)
         speech.speakUtterances(utterances)
         settings.speechVerbosityLevel = savedSpeechVerbosityLevel
 
-        brailleRegions = []
-        [cellRegions, focusedRegion] = brailleGen.getBrailleRegions(tab)
-        brailleRegions.extend(cellRegions)
-        braille.displayRegions(brailleRegions)
-
+        braille.displayRegions(brailleGen.getBrailleRegions(tab))
 
     def getTimeForCalRow(self, row, noIncs):
         """Return a string equivalent to the time of the given row in
            the calendar day view. Each calendar row is equivalent to
-           a certain time interval (from 5 minutes upto 1 hour), with 
+           a certain time interval (from 5 minutes upto 1 hour), with
            time (row 0) starting at 12 am (midnight).
 
         Arguments:
         - row: the row number.
-        - noIncs: the number of equal increments that the 24 hour period 
+        - noIncs: the number of equal increments that the 24 hour period
                   is divided into.
 
         Returns the time as a string.
@@ -142,17 +115,16 @@ class Script(default.Script):
         totalMins = timeIncrements[noIncs] * row
 
         if totalMins < 720:
-            suffix = 'ay em'
+            suffix = 'A.M.'
         else:
             totalMins -= 720
-            suffix = 'pee em'
+            suffix = 'P.M.'
 
         hrs = hours[totalMins / 60]
         mins = minutes[totalMins % 60]
 
         return hrs + ' ' + mins + ' ' + suffix
 
-
     # This method tries to detect and handle the following cases:
     # 1) Mail view: current message pane: individual lines of text.
     # 2) Mail view: current message pane: "standard" mail header lines.
@@ -179,13 +151,13 @@ class Script(default.Script):
                                event,
                                event.source.toString())
 
-        # self.walkComponentHierarchy(event.source)
+        # atspi.printAncestry(event.source)
 
         # 1) Mail view: current message pane: individual lines of text.
         #
-        # When the focus is in the pane containing the lines of an 
-        # actual mail message, then, for each of those lines, we 
-        # don't want to speak "text", the role of the component that 
+        # When the focus is in the pane containing the lines of an
+        # actual mail message, then, for each of those lines, we
+        # don't want to speak "text", the role of the component that
         # currently has focus.
         #
         # The situation is determine by checking the roles of the current
@@ -208,11 +180,10 @@ class Script(default.Script):
             orca.setLocusOfFocus(event, event.source, False)
             return
 
-
         # 2) Mail view: current message pane: "standard" mail header lines.
         #
         # Check if the focus is in the From:, To:, Subject: or Date: headers
-        # of a message in the message area, and that we want to speak all of 
+        # of a message in the message area, and that we want to speak all of
         # the tables cells for that current row.
         #
         # The situation is determine by checking the roles of the current
@@ -220,19 +191,19 @@ class Script(default.Script):
         # "text", "panel" and "table cell". If we find that, then (hopefully)
         # it's a header line in the mail message.
         #
-        # For each of the table cells in the current row in the table, we 
-        # have to work our way back down the component hierarchy until we 
-        # get a component with no children. We then use the role of that 
+        # For each of the table cells in the current row in the table, we
+        # have to work our way back down the component hierarchy until we
+        # get a component with no children. We then use the role of that
         # component to determine how to speak its contents.
         #
-        # NOTE: the code assumes that there is only one child within each 
-        # component and that the final component (with no children) is of 
+        # NOTE: the code assumes that there is only one child within each
+        # component and that the final component (with no children) is of
         # role TEXT.
 
         rolesList = [rolenames.ROLE_TEXT, \
                      rolenames.ROLE_PANEL, \
                      rolenames.ROLE_TABLE_CELL]
-        if (self.readTableCellRow == True) \
+        if self.readTableCellRow \
             and (util.isDesiredFocusedItem(event.source, rolesList)):
             debug.println(debug.LEVEL_FINEST,
                       "evolution.onFocus - mail view: current message pane: " \
@@ -256,34 +227,33 @@ class Script(default.Script):
                         result = atspi.getTextLineAtCaret(cell)
                         utterances.append(result[0])
 
-                braille.displayRegions(regions)
+                braille.displayRegions([regions, regions[0]])
                 speech.speakUtterances(utterances)
                 orca.setLocusOfFocus(event, event.source, False)
                 return
 
-
         # 3) Mail view: message header list
         #
         # Check if the focus is in the message header list. If this focus
         # event is for a different row that the last time we got a similar
         # focus event, we want to speak all of the tables cells (and the
-        # header for the one that currently has focus) in the current 
-        # highlighted message. (The role is only spoken/brailled for the 
+        # header for the one that currently has focus) in the current
+        # highlighted message. (The role is only spoken/brailled for the
         # table cell that currently has focus).
         #
         # If this focus event is just for a different table cell on the same
         # row as last time, then we just speak the current cell (and its
-        # header). 
+        # header).
         #
         # The braille cursor to set to point to the current cell.
         #
-        # Note that the Evolution user can adjust which columns appear in 
-        # the message list and the order in which they appear, so that 
+        # Note that the Evolution user can adjust which columns appear in
+        # the message list and the order in which they appear, so that
         # Orca will just speak the ones that they are interested in.
 
         rolesList = [rolenames.ROLE_TABLE_CELL, \
                      rolenames.ROLE_TREE_TABLE]
-        if (self.readTableCellRow == True) \
+        if self.readTableCellRow \
             and (util.isDesiredFocusedItem(event.source, rolesList)):
             debug.println(debug.LEVEL_FINEST,
                       "evolution.onFocus - mail view: message header list.")
@@ -303,24 +273,23 @@ class Script(default.Script):
                        ((row == 0 or row == parent.table.nRows-1) and \
                         self.lastMessageColumn == column)
 
-            savedBrailleVerbosityLevel = \
-                settings.getSetting(settings.BRAILLE_VERBOSITY_LEVEL)
-            savedSpeechVerbosityLevel = \
-                settings.getSetting(settings.SPEECH_VERBOSITY_LEVEL)
+            savedBrailleVerbosityLevel = settings.brailleVerbosityLevel
+            savedSpeechVerbosityLevel = settings.speechVerbosityLevel
+
             brailleRegions = []
             cellWithFocus = None
 
             # If the current locus of focus is not a table cell, then we
             # are entering the mail message header list (rather than moving
-            # around inside it), so speak/braille the number of mail messages 
+            # around inside it), so speak/braille the number of mail messages
             # total.
             #
             # This code section handles one other bogusity. As Evolution is
             # initially being rendered on the screen, the focus at some point
             # is given to the highlighted row in the mail message header list.
-            # Because of this, self.lastMessageRow and self.lastMessageColumn 
-            # will be set to that row number and column number, making the 
-            # setting of the speakAll variable above, incorrect. We fix that 
+            # Because of this, self.lastMessageRow and self.lastMessageColumn
+            # will be set to that row number and column number, making the
+            # setting of the speakAll variable above, incorrect. We fix that
             # up here.
 
             if orca.locusOfFocus.role != rolenames.ROLE_TABLE_CELL:
@@ -403,7 +372,7 @@ class Script(default.Script):
                             speech.speakUtterances(utterances)
 
             if brailleRegions != []:
-                braille.displayRegionsWithFocus(brailleRegions, cellWithFocus)
+                braille.displayRegions([brailleRegions, cellWithFocus])
 
             orca.setLocusOfFocus(event, event.source, False)
             settings.brailleVerbosityLevel = savedBrailleVerbosityLevel
@@ -412,7 +381,6 @@ class Script(default.Script):
             self.lastMessageRow = row
             return
 
-
         # 4) Calendar view: day view: tabbing to day with appts.
         #
         # If the focus is in the Calendar Day View on an appointment, then
@@ -420,12 +388,12 @@ class Script(default.Script):
         # date and appointment summary from the parent. This is then followed
         # by getting the information on the current appointment.
         #
-        # The start time for the appointment is determined by detecting the 
-        # equivalent child in the parent Calendar View's table has the same 
+        # The start time for the appointment is determined by detecting the
+        # equivalent child in the parent Calendar View's table has the same
         # y position on the screen.
         #
         # The end time for the appointment is determined by using the height
-        # of the current appointment component divided by the height of a 
+        # of the current appointment component divided by the height of a
         # single child in the parent Calendar View's table
         #
         # Both of these time values depend upon the value of a time increment
@@ -439,12 +407,10 @@ class Script(default.Script):
                       "evolution.onFocus - calendar view: day view: " \
                       + "tabbing to day with appts.")
 
-            brailleRegions = []
             parent = event.source.parent
             utterances = speechGen.getSpeech(parent, False)
-            [cellRegions, focusedRegion] = \
+            [brailleRegions, focusedRegion] = \
                     brailleGen.getBrailleRegions(parent)
-            brailleRegions.extend(cellRegions)
             speech.speakUtterances(utterances)
 
             apptExtents = event.source.component.getExtents(0)
@@ -476,14 +442,14 @@ class Script(default.Script):
                                 self.getTimeForCalRow(j + apptLen, noRows)
                             brailleRegions.append(braille.Region(endTime))
                             speech.speak(endTime)
-                            braille.displayRegions(brailleRegions)
+                            braille.displayRegions([brailleRegions,
+                                                    brailleRegions[0]])
                             return
 
-
         # 5) Calendar view: day view: moving with arrow keys.
         #
         # If the focus is in the Calendar Day View, check to see if there
-        # are any appointments starting at the current time. If there are, 
+        # are any appointments starting at the current time. If there are,
         # then provide the user with useful feedback for that appointment,
         # otherwise output the current time and state that there are no
         # appointments.
@@ -538,7 +504,8 @@ class Script(default.Script):
                             self.getTimeForCalRow(index + apptLen, noRows)
                         brailleRegions.append(braille.Region(endTime))
                         speech.speak(endTime)
-                        braille.displayRegions(brailleRegions)
+                        braille.displayRegions([brailleRegions,
+                                                brailleRegions[0]])
                         found = True
 
             if found == False:
@@ -549,7 +516,8 @@ class Script(default.Script):
                 utterance = _("No appointments")
                 speech.speak(utterance)
                 brailleRegions.append(braille.Region(utterance))
-                braille.displayRegions(brailleRegions)
+                braille.displayRegions([brailleRegions,
+                                        brailleRegions[0]])
 
             return
 
@@ -590,13 +558,12 @@ class Script(default.Script):
                 for i in range(0, parent.childCount):
                     child = parent.child(i)
                     if (child.role == rolenames.ROLE_PAGE_TAB_LIST):
-                        list = child
-                        tab = list.child(index-1)
+                        tabList = child
+                        tab = tabList.child(index-1)
                         if (tab.role == rolenames.ROLE_PAGE_TAB):
                             self.readPageTab(tab)
                             return
 
-
         # 7) Mail view: insert attachment dialog: unlabelled arrow button.
         #
         # Check if the focus is on the unlabelled arrow button near the
@@ -623,10 +590,10 @@ class Script(default.Script):
             utterance = _("Directories button")
             speech.speak(utterance)
             brailleRegions.append(braille.Region(utterance))
-            braille.displayRegions(brailleRegions)
+            braille.displayRegions([brailleRegions,
+                                    brailleRegions[0]])
             return
 
-
         # 8) Mail compose window: message area
         #
         # This works in conjunction with code in section 9). Check to see if
@@ -649,7 +616,6 @@ class Script(default.Script):
 
             self.message_panel = event.source.parent.parent
 
-
         # 9) Spell Checking Dialog
         #
         # This works in conjunction with code in section 8). Check to see if
@@ -671,9 +637,10 @@ class Script(default.Script):
             debug.println(debug.LEVEL_FINEST,
                       "evolution.onFocus - spell checking dialog.")
 
-            # Speak/braille the default action for this component.
+            # Braille the default action for this component.
             #
-            default.Script.onFocus(self, event)
+            orca.setLocusOfFocus(event, event.source, False)
+            self.updateBraille(orca.locusOfFocus)
 
             # Look for the "Suggestions for 'xxxxx' label in the spell
             # checker dialog panel. Extract out the xxxxx. This will be the
@@ -705,32 +672,13 @@ class Script(default.Script):
                     tokens = text.split()
                     allTokens += tokens
 
-                # Create an utterance to speak consisting of the misspelt
-                # word plus the context where it is used (upto five words 
-                # to either side of it).
-                #
-                for i in range(0, len(allTokens)):
-                    if allTokens[i].startswith(badWord):
-                        min = i - 5
-                        if min < 0:
-                            min = 0
-                        max = i + 5
-                        if max > (len(allTokens) - 1):
-                            max = len(allTokens) - 1
-
-                        utterances = [_("Misspelled word: "), badWord, \
-                                  _(" Context is ")] + allTokens[min:max+1]
-
-                        # Turn the list of utterances into a string.
-                        text = " ".join(utterances)
-                        speech.speak(text)
-                        return
-
+                util.speakMisspeltWord(allTokens, badWord)
+                return
 
         # 10) Mail view: message area - attachments.
         #
         # Check if the focus is on the "go forward" button or the
-        # "attachment button" for an attachment in the mail message 
+        # "attachment button" for an attachment in the mail message
         # attachment area. (There will be a pair of these buttons
         # for each attachment in the mail message).
         #
@@ -763,8 +711,7 @@ class Script(default.Script):
             speech.speak(utterance)
             return
 
-
-        # For everything else, pass the focus event onto the parent class 
+        # For everything else, pass the focus event onto the parent class
         # to be handled in the default way.
         #
         # Note that this includes table cells if we only want to read the
@@ -772,7 +719,6 @@ class Script(default.Script):
 
         default.Script.onFocus(self, event)
 
-
 # Values used to construct a time string for calendar appointments.
 #
 timeIncrements = {}
