@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2004-2005 Sun Microsystems Inc.
+# Copyright 2004-2006 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -39,8 +39,7 @@ def getSpeechServerFactories():
 
     factories = []
 
-    moduleNames = settings.getSetting(settings.SPEECH_FACTORY_MODULES, [])
-
+    moduleNames = settings.speechFactoryModules
     for moduleName in moduleNames:
         try:
             module =  __import__(moduleName,
@@ -65,7 +64,7 @@ def init():
     # thus we look first in the global name space, and
     # then we look in the orca namespace.
     #
-    moduleName = settings.getSetting(settings.SPEECH_SERVER_FACTORY, None)
+    moduleName = settings.speechServerFactory
 
     debug.println(debug.LEVEL_CONFIGURATION,
                   "Using speech server factory: %s" % moduleName)
@@ -88,7 +87,7 @@ def init():
 
     # Now, get the speech server we care about.
     #
-    speechServerInfo = settings.getSetting(settings.SPEECH_SERVER_INFO, None)
+    speechServerInfo = settings.speechServerInfo
     if speechServerInfo:
         __speechserver = factory.SpeechServer.getSpeechServer(speechServerInfo)
     else:
@@ -98,13 +97,13 @@ def __resolveACSS(acss=None):
     if acss:
         return acss
     else:
-        voices = settings.getSetting(settings.VOICES, None)
+        voices = settings.voices
         return voices[settings.DEFAULT_VOICE]
 
 def sayAll(utteranceIterator, progressCallback):
     if __speechserver:
         __speechserver.sayAll(utteranceIterator, progressCallback)
-    
+
 def speak(text, acss=None):
     if __speechserver:
         __speechserver.speak(text, __resolveACSS(acss))
@@ -156,7 +155,7 @@ def test():
         infos = factory.SpeechServer.getSpeechServerInfos()
         for info in infos:
             try:
-	        server = factory.SpeechServer.getSpeechServer(info)
+                server = factory.SpeechServer.getSpeechServer(info)
                 print "    ", server.getInfo()
                 for family in server.getVoiceFamilies():
                     name = family[speechserver.VoiceFamily.NAME]
@@ -165,6 +164,6 @@ def test():
                     server.speak(name, acss)
                     server.speak("testing")
                 server.shutdown()
-	    except:
-		debug.printException(debug.LEVEL_OFF)
-		pass
+            except:
+                debug.printException(debug.LEVEL_OFF)
+                pass
